package pwc.taxtech.atms.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;
import pwc.taxtech.atms.dto.ApiResultDto;
import pwc.taxtech.atms.dto.ebsdto.*;
import pwc.taxtech.atms.service.EbsApiService;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Collections;
import java.util.List;

@RestController
@RequestMapping("/ebs/api/v1/dd")
public class EbsApiController {

    private static final Logger logger = LoggerFactory.getLogger(EbsApiController.class);
    @Resource
    private EbsApiService ebsApiService;

    @RequestMapping(value = "/updateJE", method = RequestMethod.POST)
    public @ResponseBody
    ApiResultDto updateJE(@RequestParam(value="id",defaultValue = "123123123") Long id,@RequestBody List<JournalEntryQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the updateJE return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateJE(id,items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("updateJE error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/updateTB", method = RequestMethod.POST)
    public @ResponseBody ApiResultDto updateTB(@RequestParam("id") Long id,@RequestBody List<TrialBalanceQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the updateTB return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateTB(id,items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("updateTB error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/updateCF", method = RequestMethod.POST)
    public @ResponseBody ApiResultDto updateCF(@RequestParam("id") Long id,@RequestBody List<CashFlowQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the updateCF return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateCF(id,items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("updateCF error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/updateBS", method = RequestMethod.POST)
    public @ResponseBody ApiResultDto updateBS(@RequestParam("id") Long id,@RequestBody List<BalanceSheetQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the updateBS return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateBS(id,items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("updateBS error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/updatePL", method = RequestMethod.POST)
    public @ResponseBody ApiResultDto updatePL(@RequestParam("id") Long id,@RequestBody List<ProfitLossStatementQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the updatePL return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdatePL(id,items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("updatePL error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/updateBSprc", method = RequestMethod.POST)
    public @ResponseBody ApiResultDto updateBSprc(@RequestParam("id") Long id,@RequestBody List<BalanceSheetPrcQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the updateBSprc return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateBSprc(id,items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("updateBSprc error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/updatePLprc", method = RequestMethod.POST)
    public @ResponseBody ApiResultDto updatePLprc(@RequestParam("id") Long id,@RequestBody List<ProfitLossStatementPrcQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the updatePLprc return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdatePLprc(id,items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("updatePLprc error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/updateOAR", method = RequestMethod.POST)
    public @ResponseBody ApiResultDto updateOAR(@RequestParam("id") Long id,@RequestBody List<OrganizationAccountingRateQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the updateOAR return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateOAR(id,items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("updateOAR error.", e);
        }
        return ApiResultDto.fail();
    }
    @RequestMapping(value = "/updateOrg", method = RequestMethod.POST)
    public ApiResultDto updateOrg(@RequestParam("id") Long id,@RequestBody @Valid List<OrganizationQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the updateOrg return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateOrg(id,items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("updateOrg error.", e);
        }
        return ApiResultDto.fail();
    }

}
