package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.PageInfo;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.message.ErrorMessage;
import pwc.taxtech.atms.constant.enums.ReportFileUploadEnum;
import pwc.taxtech.atms.dao.ProjectMapper;
import pwc.taxtech.atms.dao.UserMapper;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileIUploadParam;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileUploadDetailResult;
import pwc.taxtech.atms.dto.reportFileUpload.ReportFileUploadParam;
import pwc.taxtech.atms.dto.reportFileUpload.ReportFileUploadResult;
import pwc.taxtech.atms.entity.Project;
import pwc.taxtech.atms.entity.User;
import pwc.taxtech.atms.exception.ServiceException;
import pwc.taxtech.atms.vat.dao.ReportFileUploadMapper;
import pwc.taxtech.atms.vat.entity.FileUpload;
import pwc.taxtech.atms.vat.entity.ReportFileUpload;
import pwc.taxtech.atms.vat.entity.ReportFileUploadExample;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class ReportFileUploadService extends BaseService {

    @Resource
    private ReportFileUploadMapper reportFileUploadMapper;

    @Autowired
    protected AuthUserHelper authUserHelper;

    @Resource
    private UserMapper userMapper;

    @Resource
    private ProjectMapper projectMapper;


    @Autowired
    DidiFileUploadService didiFileUploadService;

    public List<ReportFileUploadResult> queryData(ReportFileUploadParam param) {
        ReportFileUploadExample example = new ReportFileUploadExample();
        example.createCriteria().andProjectIdEqualTo(param.getProjectId()).andPeriodEqualTo(param.getPeriod());
        List<ReportFileUpload> dataList = reportFileUploadMapper.selectByExample(example);
        List<String> fileUploadUids = dataList.stream()
                .map(o -> o.getFileUploadId()).collect(Collectors.toList());
        Map<String, String> urlMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(fileUploadUids)) {
            DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
            fileParam.setUuids(fileUploadUids);
            PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
            if (CollectionUtils.isNotEmpty(uploadDetail.getList())) {
                urlMap = uploadDetail.getList().stream().collect(Collectors.toMap(DidiFileUploadDetailResult::getUid, didiFileUploadDetailResult -> didiFileUploadDetailResult.getViewHttpUrl()));
            }
        }
        List<String> reportTypeList = dataList.stream()
                .map(o -> o.getReportType()).collect(Collectors.toList());
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.NORMAL_DECLARATION.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.NORMAL_DECLARATION.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CORR_DECLARATION.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CORR_DECLARATION.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.NORMAL_TAX_RECEIPT.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.NORMAL_TAX_RECEIPT.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CORR_TAX_RECEIPT.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CORR_TAX_RECEIPT.getCode());
            dataList.add(upload);
        }
        List<ReportFileUploadResult> resultList = new ArrayList<ReportFileUploadResult>();
        for (ReportFileUpload data : dataList) {
            ReportFileUploadResult result = beanUtil.copyProperties(data, new ReportFileUploadResult());
            result.setFileUrl(urlMap.get(data.getFileUploadId()));
            resultList.add(result);
        }
        return resultList;
    }

    public void saveData(MultipartFile file, ReportFileUpload data) {
        if (!ReportFileUploadEnum.ReportType.MAPPING.containsKey(data.getReportType())) {
            throw new ServiceException(ErrorMessage.ParamError);
        }
        if (!ReportFileUploadEnum.SuorceType.MAPPING.containsKey(data.getSourceType())) {
            throw new ServiceException(ErrorMessage.ParamError);
        }
        String uid = authUserHelper.getCurrentUserId();
        User user = userMapper.selectByPrimaryKey(uid);
        data.setCreator(user.getUserName());
        Project project = projectMapper.selectByPrimaryKey(data.getProjectId());
        data.setOrgId(project.getOrganizationId());
        ReportFileUploadExample example = new ReportFileUploadExample();
        example.createCriteria().andProjectIdEqualTo(data.getProjectId()).andPeriodEqualTo(data.getPeriod()).andReportTypeEqualTo(data.getReportType());
        FileUpload fileUpload = didiFileUploadService.uploadFile(file, file.getOriginalFilename(), "ReportUpload");
        data.setFileUploadId(fileUpload.getUid());
        data.setUid(CommonUtils.getUUID());
        data.setCreateTime(new Date());
        data.setReportFileName(file.getOriginalFilename());
        int updateRow = reportFileUploadMapper.updateByExample(data, example);
        if (updateRow == 0) {
            reportFileUploadMapper.insert(data);
        }
    }

}
